@@ -23,9 +23,9 @@ from klotho.tonos.pitch.pitch import Pitch
 from klotho.tonos.scales.scale import Scale, AddressedScale
 from klotho.tonos.chords.chord import Chord, AddressedChord
 
-DEFAULT_DRUM_NOTE = 60
+DEFAULT_DRUM_NOTE = 75
 PERCUSSION_CHANNEL = 9
-DEFAULT_VELOCITY = 100
+DEFAULT_VELOCITY = 110
 TICKS_PER_BEAT = 480
 
 # Exact Microtonal Configuration
@@ -157,18 +157,22 @@ def _create_midi_from_temporal_unit(temporal_unit):
         if event_type == 'note_on':
             track.append(Message('note_on', 
                                channel=PERCUSSION_CHANNEL, 
-                               note=60, 
+                               note=DEFAULT_DRUM_NOTE, 
                                velocity=DEFAULT_VELOCITY, 
                                time=delta_ticks))
         else:
             track.append(Message('note_off', 
                                channel=PERCUSSION_CHANNEL, 
-                               note=60, 
+                               note=DEFAULT_DRUM_NOTE, 
                                velocity=0, 
                                time=delta_ticks))
         
         current_time = event_time
     
+    # Ensure MIDI file duration matches the TemporalUnit's total duration
+    # This handles trailing rests that would otherwise be cut off
+    _ensure_midi_duration(track, temporal_unit.duration, bpm)
+    
     return midi_file
 
 def _create_midi_from_compositional_unit(compositional_unit):
@@ -243,6 +247,10 @@ def _create_midi_from_compositional_unit(compositional_unit):
     # Use the exact same event processing as pitch collections
     _events_to_midi_messages(events, track, bpm)
     
+    # Ensure MIDI file duration matches the CompositionalUnit's total duration
+    # This handles trailing rests that would otherwise be cut off
+    _ensure_midi_duration(track, compositional_unit.duration, bpm)
+    
     return midi_file
 
 def _create_midi_from_collection(collection):
@@ -263,6 +271,9 @@ def _create_midi_from_collection(collection):
     
     track.append(MetaMessage('set_tempo', tempo=int(60_000_000 / bpm)))
     
+    # Reset microtonal channel counter for this new file
+    _reset_microtonal_counter()
+    
     # Collect all events from all units in the collection
     all_events = []
     
@@ -332,13 +343,22 @@ def _create_midi_from_collection(collection):
         
         current_time = event_time
     
+    # Ensure MIDI file duration matches the collection's total duration
+    # This handles trailing rests that would otherwise be cut off
+    if isinstance(collection, TemporalUnitSequence):
+        total_duration = collection.duration
+        _ensure_midi_duration(track, total_duration, bpm)
+    
     return midi_file
 
 def _collect_events_from_unit_with_offset(unit, all_events, time_offset=0.0):
     """Helper function to collect events from a single temporal unit with time offset."""
-    if isinstance(unit, CompositionalUnit):
+    # Create a temporary copy to avoid modifying the original unit's state
+    unit_copy = unit.copy()
+    
+    if isinstance(unit_copy, CompositionalUnit):
         # CompositionalUnit with parameters
-        for event in unit:
+        for event in unit_copy:
             if not event.is_rest:
                 # Get instrument information from the parameter tree
                 instrument = event._pt.get_active_instrument(event._node_id)
@@ -394,7 +414,7 @@ def _collect_events_from_unit_with_offset(unit, all_events, time_offset=0.0):
                     all_events.append((start_time + duration, 'note_off', channel, midi_note, 0, program))
     else:
         # Regular TemporalUnit - use defaults
-        for chronon in unit:
+        for chronon in unit_copy:
             if not chronon.is_rest:
                 start_time = chronon.start + time_offset
                 duration = abs(chronon.duration)
@@ -444,6 +464,7 @@ def _midi_to_audio_colab(midi_path, audio_path):
         subprocess.run([
             'timidity', midi_path, 
             '-Ow', '-o', audio_path,
+            '--preserve-silence',  # Prevent dropping initial/trailing rests
             '--quiet'
         ], check=True, capture_output=True)
         
@@ -451,8 +472,21 @@ def _midi_to_audio_colab(midi_path, audio_path):
         return audio_widget
         
     except (subprocess.CalledProcessError, FileNotFoundError) as e:
-        print("Timidity not found. Install it first with: !apt install timidity fluid-soundfont-gm")
-        return _midi_to_audio_fallback(midi_path)
+        # If --preserve-silence option failed, try without it
+        try:
+            print("--preserve-silence option not supported, trying without it...")
+            subprocess.run([
+                'timidity', midi_path, 
+                '-Ow', '-o', audio_path,
+                '--quiet'
+            ], check=True, capture_output=True)
+            
+            audio_widget = Audio(audio_path, autoplay=False)
+            return audio_widget
+            
+        except (subprocess.CalledProcessError, FileNotFoundError) as e2:
+            print("Timidity not found. Install it first with: !apt install timidity fluid-soundfont-gm")
+            return _midi_to_audio_fallback(midi_path)
 
 def _midi_to_audio_fluidsynth(midi_path, audio_path):
     """Convert MIDI to audio using FluidSynth (original method)."""
@@ -540,6 +574,48 @@ def _reset_microtonal_counter():
     if hasattr(_get_microtonal_channel_and_note, '_channel_counter'):
         _get_microtonal_channel_and_note._channel_counter = 1
 
+def _ensure_midi_duration(track, target_duration_seconds, bpm):
+    """
+    Ensure MIDI track duration matches target duration by adding silent padding if needed.
+    
+    This handles cases where the temporal structure ends with rests, ensuring
+    the MIDI file doesn't cut off prematurely and allows reverb/sustain to play out.
+    
+    Parameters
+    ----------
+    track : MidiTrack
+        The MIDI track to extend
+    target_duration_seconds : float
+        The target duration in seconds
+    bpm : float
+        Beats per minute for timing calculations
+    """
+    if not track:
+        return
+    
+    # Calculate current track duration
+    current_time_ticks = 0
+    for msg in track:
+        current_time_ticks += msg.time
+    
+    # Convert to seconds
+    beat_duration = 60.0 / bpm
+    current_duration_seconds = current_time_ticks * beat_duration / TICKS_PER_BEAT
+    
+    # If we need more duration, add a silent padding message
+    if target_duration_seconds > current_duration_seconds:
+        missing_duration = target_duration_seconds - current_duration_seconds
+        missing_ticks = int(missing_duration / beat_duration * TICKS_PER_BEAT)
+        
+        # Add a silent message (could be a control change or just a dummy note off)
+        # Using a note off with velocity 0 on a silent channel as padding
+        from mido import Message
+        track.append(Message('note_off', 
+                           channel=15,  # Use channel 15 for silent padding
+                           note=127,    # High note that won't interfere
+                           velocity=0, 
+                           time=missing_ticks))
+
 def _create_midi_from_pitch_collection(collection, dur=0.5, bpm=120, prgm=0):
     """Create a MIDI file from a PitchCollection (sequential playback)."""
     midi_file = MidiFile(ticks_per_beat=TICKS_PER_BEAT)
